import { HttpClient, HttpErrorResponse } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { catchError, map, Observable, of } from 'rxjs';
import { LoginResponse, RegisterResponse } from './responses/auth';

@Injectable({
  providedIn: 'root'
})
export class AuthService {
  private readonly API_BASE: string = "http://localhost:5000"

  private readonly USERNAME_FIELD: string = "username";
  private readonly PASSWORD_FIELD: string = "password";
  private readonly REPEAT_PASSWORD_FIELD: string = "repeatPassword";

  constructor(private http: HttpClient) { }

  public Login(username: string, password: string): Observable<LoginResponse> {
    let url = `${this.API_BASE}/auth/login`;

    let formData = new FormData();
    formData.append(this.USERNAME_FIELD, username);
    formData.append(this.PASSWORD_FIELD, password);

    return this.http.post<LoginResponse>(url, formData, { withCredentials: true })
  }

  public Register(username: string, password: string, repeatPassword: string): Observable<RegisterResponse> {
    let url = `${this.API_BASE}/auth/register`;

    let formData = new FormData();
    formData.append(this.USERNAME_FIELD, username);
    formData.append(this.PASSWORD_FIELD, password);
    formData.append(this.REPEAT_PASSWORD_FIELD, repeatPassword);

    return this.http.post<RegisterResponse>(url, formData, { withCredentials: true })
  }

  public Logout(): Observable<boolean> {
    let url = `${this.API_BASE}/auth/logout`;

    return this.http.get(url, { withCredentials: true }).pipe(
      map(() => true),
      catchError(() => of(false))
    );
  }

  public Bump(): Observable<boolean> {
    let url = `${this.API_BASE}/auth/bump`

    return this.http.get(url, { withCredentials: true }).pipe(
      map(() => true),
      catchError(() => of(false))
    );
  }
}
